// src/index.ts
var externalRE = /^(https?:)?\/\//;
var isExternalUrl = (url) => externalRE.test(url);
var exclude = [];
var tempOptions = {};
function externalsExtensionResolverConfigFilter() {
  return {
    name: "vite-plugin-externals-extension-config-filter",
    enforce: "post" /* POST */,
    async config(config) {
      var _a, _b;
      const include = (_b = (_a = config.optimizeDeps) == null ? void 0 : _a.include) != null ? _b : [];
      include == null ? void 0 : include.forEach((url, index) => {
        if (exclude.includes(url)) {
          include.splice(index, 1);
        }
      });
    }
  };
}
async function getExcludeUrls(options) {
  if (exclude.length) {
    return exclude;
  }
  exclude.push(
    ...await Promise.all(
      Object.entries(options).filter((option) => !option[1].getter).map(async (option) => {
        option[1].url = typeof option[1].url === "function" ? await option[1].url() : option[1].url;
        return option[1].url;
      })
    )
  );
  return exclude;
}
function externalsExtensionResolver(options) {
  tempOptions = options;
  return {
    name: "vite-plugin-externals-extension",
    enforce: "pre" /* PRE */,
    async resolveId(id) {
      if (options[id] && !!options[id].getter) {
        return id;
      } else if (options[id]) {
        return options[id].url;
      }
      return null;
    },
    async config() {
      return {
        build: {
          rollupOptions: {
            external: await getExcludeUrls(tempOptions)
          }
        }
      };
    },
    async transformIndexHtml() {
      return Object.entries(options).filter((option) => !!option[1].getter).map((option) => option[1].url).filter(isExternalUrl).map((url) => ({
        tag: "script",
        attrs: {
          src: url
        }
      }));
    },
    async load(id) {
      if (options[id]) {
        const { getter } = options[id] || {};
        if (typeof getter === "function") {
          return `
            const getter = ${getter};
            const fetchGetterModule = getter(window);
            export default fetchGetterModule;
          `;
        }
        return `
          export default window['${getter}']
        `;
      }
      return null;
    }
  };
}
async function compatLowVersion() {
  return await getExcludeUrls(tempOptions);
}
function externalsExtension(options) {
  return [externalsExtensionResolver(options), externalsExtensionResolverConfigFilter()];
}
export {
  compatLowVersion,
  externalsExtension
};
//# sourceMappingURL=index.mjs.map